﻿using System.ComponentModel.DataAnnotations;
using Luticate2.Utils.Dbo.Basic;
using Luticate2.Utils.Dbo.PaginatedRequest;
using Luticate2.Utils.Interfaces;
using Luticate2.Utils.Utils;
using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Options;

namespace Luticate2.Utils.Controllers
{
    public abstract class LuCrudController<TBusiness, TDboCreate, TDboRead, TDboUpdate, TId> : LuController
        where TDboCreate : class
        where TDboRead : class
        where TDboUpdate : class
        where TBusiness : ILuCrudInterface<TDboCreate, TDboRead, TDboUpdate, TId>
    {
        protected readonly TBusiness Business;

        protected LuCrudController(TBusiness business, IOptions<LuUtilsOptionsDbo> luUtilsOptionsDbo) : base(luUtilsOptionsDbo)
        {
            Business = business;
        }

        [HttpGet]
        [Route("[controller]/{id}")]
        public virtual LuApiWrapperDbo<TDboRead> GetSingleById([Required]TId id)
        {
            return Handle(Business.GetSingleById(id));
        }

        [HttpGet]
        [Route("[controller]")]
        public virtual LuApiWrapperDbo<LuPaginatedDbo<TDboRead>> GetMultiple([Required]LuPaginatedRequestDbo request)
        {
            return Handle(Business.GetMultiple(request));
        }

        [HttpPost]
        [Route("[controller]")]
        public virtual LuApiWrapperDbo<TDboRead> AddDbo([FromBody][Required]TDboCreate data)
        {
            return Handle(Business.AddDbo(data));
        }

        [HttpPost]
        [Route("[controller]/{id}")]
        public virtual LuApiWrapperDbo<TDboRead> EditSingleByIdDbo([Required]TId id, [FromBody][Required]TDboUpdate data)
        {
            return Handle(Business.EditSingleByIdDbo(id, data));
        }

        [HttpDelete]
        [Route("[controller]/{id}")]
        public virtual LuApiWrapperDbo<TDboRead> Delete([Required]TId id)
        {
            return Handle(Business.DeleteSingleByIdDbo(id));
        }
    }
}